@@ -2,120 +2,9 @@
 # -*- coding: utf-8 -*-
 
 import itertools as it, operator as op, functools as ft
-from collections import Mapping, OrderedDict
+from lya import AttrDict, configure_logging
+from collections import OrderedDict
 import os, sys
-import yaml, yaml.constructor
-
-
-class OrderedDictYAMLLoader(yaml.Loader):
-	'Based on: https://gist.github.com/844388'
-
-	def __init__(self, *args, **kwargs):
-		yaml.Loader.__init__(self, *args, **kwargs)
-		self.add_constructor(u'tag:yaml.org,2002:map', type(self).construct_yaml_map)
-		self.add_constructor(u'tag:yaml.org,2002:omap', type(self).construct_yaml_map)
-
-	def construct_yaml_map(self, node):
-		data = OrderedDict()
-		yield data
-		value = self.construct_mapping(node)
-		data.update(value)
-
-	def construct_mapping(self, node, deep=False):
-		if isinstance(node, yaml.MappingNode):
-			self.flatten_mapping(node)
-		else:
-			raise yaml.constructor.ConstructorError( None, None,
-				'expected a mapping node, but found {}'.format(node.id), node.start_mark )
-
-		mapping = OrderedDict()
-		for key_node, value_node in node.value:
-			key = self.construct_object(key_node, deep=deep)
-			try:
-				hash(key)
-			except TypeError, exc:
-				raise yaml.constructor.ConstructorError( 'while constructing a mapping',
-					node.start_mark, 'found unacceptable key ({})'.format(exc), key_node.start_mark )
-			value = self.construct_object(value_node, deep=deep)
-			mapping[key] = value
-		return mapping
-
-
-class AttrDict(dict):
-
-	def __init__(self, *argz, **kwz):
-		for k,v in dict(*argz, **kwz).iteritems(): self[k] = v
-
-	def __setitem__(self, k, v):
-		super(AttrDict, self).__setitem__( k,
-			AttrDict(v) if isinstance(v, Mapping) else v )
-	def __getattr__(self, k):
-		if not k.startswith('__'): return self[k]
-		else: raise AttributeError # necessary for stuff like __deepcopy__ or __hash__
-	def __setattr__(self, k, v): self[k] = v
-
-	@classmethod
-	def from_yaml(cls, path, if_exists=False):
-		import yaml
-		if if_exists and not os.path.exists(path): return cls()
-		return cls(yaml.load(open(path), OrderedDictYAMLLoader))
-
-	@staticmethod
-	def flatten_dict(data, path=tuple()):
-		dst = list()
-		for k,v in data.iteritems():
-			k = path + (k,)
-			if isinstance(v, Mapping):
-				for v in v.flatten(k): dst.append(v)
-			else: dst.append((k, v))
-		return dst
-
-	def flatten(self, path=tuple()):
-		return self.flatten_dict(self, path=path)
-
-	def update_flat(self, val):
-		if isinstance(val, AttrDict): val = val.flatten()
-		for k,v in val:
-			dst = self
-			for slug in k[:-1]:
-				if dst.get(slug) is None:
-					dst[slug] = AttrDict()
-				dst = dst[slug]
-			if v is not None or not isinstance(
-				dst.get(k[-1]), Mapping ): dst[k[-1]] = v
-
-	def update_dict(self, data):
-		self.update_flat(self.flatten_dict(data))
-
-	def update_yaml(self, path):
-		self.update_flat(self.from_yaml(path))
-
-	def clone(self):
-		clone = AttrDict()
-		clone.update_dict(self)
-		return clone
-
-	def rebase(self, base):
-		base = base.clone()
-		base.update_dict(self)
-		self.clear()
-		self.update_dict(base)
-
-
-def configure_logging(cfg, custom_level=None):
-	import logging, logging.config
-	if custom_level is None: custom_level = logging.WARNING
-	for entity in it.chain.from_iterable(it.imap(
-			op.methodcaller('viewvalues'),
-			[cfg] + list(cfg.get(k, dict()) for k in ['handlers', 'loggers']) )):
-		if isinstance(entity, Mapping)\
-			and entity.get('level') == 'custom': entity['level'] = custom_level
-	logging.config.dictConfig(cfg)
-	logging.captureWarnings(cfg.warnings)
-	if not cfg.tracebacks:
-		class NoTBLogger(logging.Logger):
-			def exception(self, *argz, **kwz): self.error(*argz, **kwz)
-		logging.setLoggerClass(NoTBLogger)
 
 
 def main():
@@ -163,6 +52,10 @@ def main():
 			' Values from the latter ones override values in the former.'
 			' Available CLI options override the values in any config.')
 
+	parser.add_argument('-a', '--xattr-emulation', metavar='db-path',
+		help='Emulate filesystem extended attributes (used in'
+			' some collectors like sysstat or cron_log), storing per-path'
+			' data in a simple shelve db.')
 	parser.add_argument('-n', '--dry-run',
 		action='store_true', help='Do not actually send data.')
 	parser.add_argument('--debug',
@@ -174,17 +67,14 @@ def main():
 		os.path.splitext(os.path.realpath(__file__))[0] ))
 	for k in optz.config: cfg.update_yaml(k)
 
-	# Deprecated stuff
-	if cfg.get('core') or cfg.get('carbon'):
-		raise ValueError(
-			'Old-style loop/sink configuration options usage detected.'
-			' These are no longer supported or used.'
-			' Please move these into "loop" section of the configuration file.' )
-
 	# Logging
 	import logging
 	configure_logging( cfg.logging,
 		logging.DEBUG if optz.debug else logging.WARNING )
+	if not cfg.logging.tracebacks:
+		class NoTBLogger(logging.Logger):
+			def exception(self, *argz, **kwz): self.error(*argz, **kwz)
+		logging.setLoggerClass(NoTBLogger)
 	log = logging.getLogger(__name__)
 
 	# Fill "auto-detected" blanks in the configuration, CLI overrides
@@ -198,6 +88,22 @@ def main():
 	except KeyError: pass
 	if optz.interval: cfg.loop.interval = optz.interval
 	if optz.dry_run: cfg.debug.dry_run = optz.dry_run
+	if optz.xattr_emulation: cfg.core.xattr_emulation = optz.xattr_emulation
+
+	# Fake "xattr" module, if requested
+	if cfg.core.xattr_emulation:
+		import shelve
+		xattr_db = shelve.open(cfg.core.xattr_emulation, 'c')
+		class xattr_path(object):
+			def __init__(self, base):
+				assert isinstance(base, str)
+				self.base = base
+			def key(self, k): return '{}\0{}'.format(self.base, k)
+			def __setitem__(self, k, v): xattr_db[self.key(k)] = v
+			def __getitem__(self, k): return xattr_db[self.key(k)]
+			def __del__(self): xattr_db.sync()
+		class xattr_module(object): xattr = xattr_path
+		sys.modules['xattr'] = xattr_module
 
 	# Override "enabled" collector/sink parameters, based on CLI
 	ep_conf = dict()
@@ -222,7 +128,8 @@ def main():
 		conf_base, conf, objects, enabled, disabled = ep_conf[ep_key]
 		ep_dict = dict( (ep.name, ep) for ep in
 			pkg_resources.iter_entry_points('graphite_metrics.{}'.format(ep_key)) )
-		eps = OrderedDict( (name, (ep_dict.pop(name), subconf))
+		eps = OrderedDict(
+			(name, (ep_dict.pop(name), subconf or AttrDict()))
 			for name, subconf in conf.viewitems() if name in ep_dict )
 		eps.update( (name, (module, conf_base))
 			for name, module in ep_dict.viewitems() )
